#include "Compiler.h"
#include "HardwareProfile.h"
#include <plib.h>

#include "LCD.h"

// Define constants for PINS
#define LCD_DB4		LATDbits.LATD0
#define LCD_DB5		LATDbits.LATD1
#define LCD_DB6		LATDbits.LATD2
#define LCD_DB7		LATDbits.LATD3

#define LCD_RS    	LATDbits.LATD4
//#define LCD_RW    	LATDbits.LATD9
#define LCD_E     	LATDbits.LATD10

short const LCD_INIT_STRING[4] =
{
 0x20 | (lcd_type << 2), // Func set: 4-bit, 2 lines, 5x8 dots
 0xc,                    // Display on
 1,                      // Clear display
 6                       // Increment cursor
 };

//----------------------------
void lcd_init(void)
{
	short i;
	
	//Set D0-4,D9-10 as a digital output, change if you change the pins
	
	LATD |= 0x061F; TRISD &= 0xF9E0;
	// works fine off 3.3V, in fact the following line kills the timing
	//mPORTDOpenDrainOpen(BIT_0 | BIT_1 | BIT_2 | BIT_3 | BIT_4 | BIT_9 | BIT_10);

	Delayms(30);

	LCD_RS = LOW; // Register Select Command Register

	//#ifdef USE_LCD_RW
	//LCD_RW = LOW; 
	//#endif

	LCD_E = LOW; // Disable

	Delayms(15);

	for(i=0; i < 3; i++)
	   {
	    lcd_send_nibble(0x03);
	    Delayms(4);
	   }
	
	lcd_send_nibble(0x02);
	
	for(i=0; i < sizeof(LCD_INIT_STRING); i++)
	   {
	    lcd_send_byte(0, LCD_INIT_STRING[i]);
	   
	    // If the R/W signal is not used, then
	    // the busy bit can't be polled.  One of
	    // the init commands takes longer than
	    // the hard-coded delay of 60 us, so in
	    // that case, lets just do a 5 ms delay
	    // after all four of them.
	    //#ifndef USE_LCD_RW
	    Delayms(5);
	    //#endif
	   }
	Delayms(5);
	putsLCD("\f");
	Delayms(5);
}

//-------------------------------------
void lcd_send_nibble(short nibble)
{
// Note:  !! converts an integer expression
// to a boolean (1 or 0).
 LCD_DB4 = !!(nibble & 1);
 LCD_DB5 = !!(nibble & 2); 
 LCD_DB6 = !!(nibble & 4);   
 LCD_DB7 = !!(nibble & 8);   

 Delaytus(1);
 LCD_E = HIGH; // Enable
 Delayus(2);
 LCD_E = LOW; // Disable
}

//----------------------------------------
// Send a byte to the LCD.
void lcd_send_byte(short address, short n)
{
	LCD_RS = LOW;

	// Need to investiage bit_test
	//#ifdef USE_LCD_RW
	//while(bit_test(lcd_read_byte(),7)) ;
	//#else
	Delayms(1); 
	//#endif

	if(address)
	   LCD_RS = HIGH;
	else
	   LCD_RS = LOW;
	     
	Delaytus(1);

	//#ifdef USE_LCD_RW
	//LCD_RW = LOW;
	//Delaytus(1);
	//#endif

	LCD_E = LOW;
	
	lcd_send_nibble(n >> 4);
	lcd_send_nibble(n & 0xf);
}

//-----------------------------
void putsLCD(char *s)
{ 	
	while( *s != '\0')
	{	
		switch(*s)
		{
			case '\f':				// Clear, go to First Line
		    {
				lcd_send_byte(0,1);
		      	Delayms(2);
		      	break;
		   	}
		    case '\n':				// Next Line
		    {
				lcd_gotoxy(1,2);
		    	break;
			}
		    case '\b':
		    {
				lcd_send_byte(0,0x10);
		    	break;
		   	}
		    default:
		    {
				lcd_send_byte(1,*s);
		    	break;
		    }
		}
		s++;
		Delayus(10);
	}
}
//----------------------------

void lcd_gotoxy(short x, short y)
{
	short address;
	
	if(y != 1)
	   address = lcd_line_two;
	else
	   address=0;
	
	address += x-1;
	lcd_send_byte(0, 0x80 | address);
}
   
//==========================

void Delayms( unsigned t)
// This uses Timer 1, can be changed to another timer. Assumes FPB = SYS_FREQ
{
    OpenTimer1(T1_ON | T1_PS_1_256, 0xFFFF);
    while (t--)
    {  // t x 1ms loop
        WriteTimer1(0);
        while (ReadTimer1() < FPB/256/1000);
	}
	CloseTimer1();
} // Delayms

void Delayus( unsigned t)
{
	OpenTimer1(T1_ON | T1_PS_1_256, 0xFFFF);
    while (t--)
    {  // t x 1ms loop
        WriteTimer1(0);
        while (ReadTimer1() < FPB/256/1000000);
	}
	CloseTimer1();
}// Delayus

void Delaytus( unsigned t)
{
	OpenTimer1(T1_ON | T1_PS_1_256, 0xFFFF);
    while (t--)
    {  // t x 1ms loop
        WriteTimer1(0);
        while (ReadTimer1() < FPB/256/10000000);
	}
	CloseTimer1();
}// Delaytus

